/*
 * Copyright 2019 Confluent Inc.
 *
 * Licensed under the Confluent Community License (the "License"); you may not use
 * this file except in compliance with the License.  You may obtain a copy of the
 * License at
 *
 * http://www.confluent.io/confluent-community-license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 */

package io.confluent.kafka.secretregistry.crypto;

import static java.util.Base64.getDecoder;
import static java.util.Base64.getEncoder;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import java.util.Base64;
import java.util.Objects;

public class Encrypted {

  public static Encrypted of(String info, byte[] content, byte[] iv) {
    Base64.Encoder encoder = getEncoder();
    String contentBase64 = encoder.encodeToString(content);
    String ivBase64 = encoder.encodeToString(iv);
    return new Encrypted(info, contentBase64, ivBase64);
  }

  @SuppressWarnings("unused")
  @JsonCreator
  public static Encrypted fromJson(
      @JsonProperty("derivationInfo") String derivationInfo,
      @JsonProperty("content") String content, @JsonProperty("iv") String iv
  ) {
    return new Encrypted(derivationInfo, content, iv);
  }

  private final String derivationInfo;
  private final String content;
  private final String iv;

  public Encrypted(String derivationInfo, String content, String iv) {
    this.derivationInfo = derivationInfo;
    this.content = content;
    this.iv = iv;
  }

  @JsonProperty
  public String derivationInfo() {
    return derivationInfo;
  }

  @JsonProperty
  public String content() {
    return content;
  }

  @JsonProperty
  public String iv() {
    return iv;
  }

  public byte[] contentBytes() {
    return getDecoder().decode(content());
  }

  public byte[] ivBytes() {
    return getDecoder().decode(iv());
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    Encrypted encrypted = (Encrypted) o;
    return Objects.equals(derivationInfo, encrypted.derivationInfo)
        && Objects.equals(content, encrypted.content)
        && Objects.equals(iv, encrypted.iv);
  }

  @Override
  public int hashCode() {
    return Objects.hash(derivationInfo, content, iv);
  }

  @Override
  public String toString() {
    return "Encrypted{"
        + "derivationInfo='" + derivationInfo + '\''
        + ", content='[hidden]', iv='[hidden]'"
        + '}';
  }
}
