/*
 * Copyright 2019 Confluent Inc.
 *
 * Licensed under the Confluent Community License (the "License"); you may not use
 * this file except in compliance with the License.  You may obtain a copy of the
 * License at
 *
 * http://www.confluent.io/confluent-community-license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 */

package io.confluent.kafka.secretregistry.rest.resources;

import java.util.List;
import java.util.Map;

import javax.validation.constraints.NotNull;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.container.AsyncResponse;
import javax.ws.rs.container.Suspended;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.SecurityContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.confluent.kafka.secretregistry.client.rest.Versions;
import io.confluent.kafka.secretregistry.client.rest.entities.Secret;
import io.confluent.kafka.secretregistry.client.rest.entities.requests.RegisterSecretRequest;
import io.confluent.kafka.secretregistry.client.rest.entities.requests.RegisterSecretResponse;
import io.confluent.kafka.secretregistry.exceptions.InvalidVersionException;
import io.confluent.kafka.secretregistry.exceptions.OperationNotPermittedException;
import io.confluent.kafka.secretregistry.exceptions.SecretRegistryException;
import io.confluent.kafka.secretregistry.exceptions.SecretRegistryRequestForwardingException;
import io.confluent.kafka.secretregistry.exceptions.SecretRegistryStoreException;
import io.confluent.kafka.secretregistry.exceptions.SecretRegistryTimeoutException;
import io.confluent.kafka.secretregistry.exceptions.UnknownMasterException;
import io.confluent.kafka.secretregistry.rest.VersionId;
import io.confluent.kafka.secretregistry.rest.exceptions.Errors;
import io.confluent.kafka.secretregistry.storage.KafkaSecretRegistry;
import io.confluent.rest.annotations.PerformanceMetric;

@Path("/secret/paths/{path}/keys/{key}/versions")
@Produces({Versions.SECRET_REGISTRY_V1_JSON_WEIGHTED,
    Versions.SECRET_REGISTRY_DEFAULT_JSON_WEIGHTED,
    Versions.JSON_WEIGHTED})
@Consumes({Versions.SECRET_REGISTRY_V1_JSON,
    Versions.SECRET_REGISTRY_DEFAULT_JSON,
    Versions.JSON, Versions.GENERIC_REQUEST})
public class PathKeyVersionResource {

  private static final Logger log = LoggerFactory.getLogger(PathKeyVersionResource.class);

  private final KafkaSecretRegistry secretRegistry;

  private final RequestHeaderBuilder requestHeaderBuilder = new RequestHeaderBuilder();

  public PathKeyVersionResource(KafkaSecretRegistry registry) {
    this.secretRegistry = registry;
  }

  @GET
  @Path("/{version}")
  @PerformanceMetric("paths.keys.versions.get-secret")
  public Secret getSecret(
      @Context SecurityContext securityContext,
      @PathParam("path") String path,
      @PathParam("key") String key,
      @PathParam("version") String version
  ) {
    if (securityContext.getUserPrincipal() != null) {
      String name = securityContext.getUserPrincipal().getName();
      log.info("User '{}' getting secret '{}:{}' version {}", name, path, key, version);
    } else {
      log.info("Unknown user getting secret '{}:{}' version {}", path, key, version);
    }

    VersionId versionId = null;
    try {
      versionId = new VersionId(version);
    } catch (InvalidVersionException e) {
      throw Errors.invalidVersionException();
    }
    Secret secret = null;
    String errorMessage = null;
    try {
      secret = secretRegistry.validateAndGetSecret(path, key, versionId);
    } catch (SecretRegistryStoreException e) {
      errorMessage =
          "Error while retrieving secret for path "
              + path
              + " with key "
              + key
              + " with version "
              + version
              + " from the secret registry";
      log.debug(errorMessage, e);
      throw Errors.storeException(errorMessage, e);
    } catch (InvalidVersionException e) {
      throw Errors.invalidVersionException();
    } catch (SecretRegistryException e) {
      throw Errors.secretRegistryException(errorMessage, e);
    }
    return secret;
  }

  @GET
  @Path("/{version}/secret")
  @PerformanceMetric("paths.keys.versions.get-secret.only")
  public String getSecretOnly(
      @Context SecurityContext securityContext,
      @PathParam("path") String path,
      @PathParam("key") String key,
      @PathParam("version") String version
  ) {
    return getSecret(securityContext, path, key, version).getSecret();
  }

  @GET
  @PerformanceMetric("paths.keys.versions.list")
  public List<Integer> list(
      @Context SecurityContext securityContext,
      @PathParam("path") String path,
      @PathParam("key") String key
  ) {
    if (securityContext.getUserPrincipal() != null) {
      String name = securityContext.getUserPrincipal().getName();
      log.info("User '{}' listing versions for '{}:{}'", name, path, key);
    } else {
      log.info("Unknown user listing versions for '{}:{}'", path, key);
    }

    // check if path exists. If not, throw 404
    String errorMessage = "Error while validating that path "
        + path + " and key " + key
        + " exist in the registry";
    try {
      if (!secretRegistry.listPaths().contains(path)) {
        throw Errors.pathNotFoundException();
      }
      if (!secretRegistry.listKeys(path).contains(key)) {
        throw Errors.keyNotFoundException();
      }
    } catch (SecretRegistryStoreException e) {
      throw Errors.storeException(errorMessage, e);
    } catch (SecretRegistryException e) {
      throw Errors.secretRegistryException(errorMessage, e);
    }
    errorMessage = "Error while listing all versions for path "
        + path + " and key " + key;
    try {
      return secretRegistry.listVersions(path, key);
    } catch (SecretRegistryStoreException e) {
      throw Errors.storeException(errorMessage, e);
    } catch (SecretRegistryException e) {
      throw Errors.secretRegistryException(errorMessage, e);
    }
  }

  @POST
  @PerformanceMetric("paths.keys.versions.register")
  public void register(
      final @Suspended AsyncResponse asyncResponse,
      @Context SecurityContext securityContext,
      @Context HttpHeaders headers,
      @PathParam("path") String path,
      @PathParam("key") String key,
      @NotNull RegisterSecretRequest request
  ) {
    if (securityContext.getUserPrincipal() != null) {
      String name = securityContext.getUserPrincipal().getName();
      log.info("User '{}' registering secret '{}:{}'", name, path, key);
    } else {
      log.info("Unknown user registering secret '{}:{}'", path, key);
    }

    Map<String, String> headerProperties = requestHeaderBuilder.buildRequestHeaders(
        headers, secretRegistry.config().whitelistHeaders());

    Secret secret = new Secret(path, key, 0, request.getSecret());
    int id;
    try {
      id = secretRegistry.registerOrForward(secret, headerProperties);
    } catch (OperationNotPermittedException e) {
      throw Errors.operationNotPermittedException(e.getMessage());
    } catch (SecretRegistryTimeoutException e) {
      throw Errors.operationTimeoutException("Register operation timed out", e);
    } catch (SecretRegistryStoreException e) {
      throw Errors.storeException("Register secret operation failed while writing"
          + " to the Kafka store", e);
    } catch (SecretRegistryRequestForwardingException e) {
      throw Errors.requestForwardingFailedException("Error while forwarding register secret request"
          + " to the master", e);
    } catch (UnknownMasterException e) {
      throw Errors.unknownMasterException("Master not known.", e);
    } catch (SecretRegistryException e) {
      throw Errors.secretRegistryException("Error while registering secret", e);
    }
    RegisterSecretResponse registerSecretResponse = new RegisterSecretResponse();
    registerSecretResponse.setVersion(id);
    asyncResponse.resume(registerSecretResponse);
  }

  @DELETE
  @Path("/{version}")
  @PerformanceMetric("paths.keys.versions.delete-secret")
  public void deleteSecretVersion(
      final @Suspended AsyncResponse asyncResponse,
      @Context SecurityContext securityContext,
      @Context HttpHeaders headers,
      @PathParam("path") String path,
      @PathParam("key") String key,
      @PathParam("version") String version
  ) {
    if (securityContext.getUserPrincipal() != null) {
      String name = securityContext.getUserPrincipal().getName();
      log.info("User '{}' deleting secret '{}:{}' version {}", name, path, key, version);
    } else {
      log.info("Unknown user deleting secret '{}:{}' version {}", path, key, version);
    }

    VersionId versionId = null;
    try {
      versionId = new VersionId(version);
    } catch (InvalidVersionException e) {
      throw Errors.invalidVersionException();
    }
    Secret secret = null;
    String errorMessage = null;
    try {
      secret = secretRegistry.validateAndGetSecret(path, key, versionId);
    } catch (SecretRegistryStoreException e) {
      errorMessage =
          "Error while retrieving secret for path "
              + path
              + " with key "
              + key
              + " with version "
              + version
              + " from the secret registry";
      log.debug(errorMessage, e);
      throw Errors.storeException(errorMessage, e);
    } catch (InvalidVersionException e) {
      throw Errors.invalidVersionException();
    } catch (SecretRegistryException e) {
      throw Errors.secretRegistryException(errorMessage, e);
    }
    try {
      Map<String, String> headerProperties = requestHeaderBuilder.buildRequestHeaders(
          headers, secretRegistry.config().whitelistHeaders());
      secretRegistry.deleteVersionOrForward(
          headerProperties, secret.getPath(), secret.getKey(), secret.getVersion());
    } catch (SecretRegistryTimeoutException e) {
      throw Errors.operationTimeoutException("Delete Secret Version operation timed out", e);
    } catch (SecretRegistryStoreException e) {
      throw Errors.storeException("Delete Secret Version operation failed while writing"
          + " to the Kafka store", e);
    } catch (SecretRegistryRequestForwardingException e) {
      throw Errors
          .requestForwardingFailedException("Error while forwarding delete secret version request"
              + " to the master", e);
    } catch (UnknownMasterException e) {
      throw Errors.unknownMasterException("Master not known.", e);
    } catch (SecretRegistryException e) {
      throw Errors.secretRegistryException("Error while deleting Secret Version", e);
    }

    asyncResponse.resume(versionId.getVersionId());
  }
}
