/*
 * Copyright 2019 Confluent Inc.
 *
 * Licensed under the Confluent Community License (the "License"); you may not use
 * this file except in compliance with the License.  You may obtain a copy of the
 * License at
 *
 * http://www.confluent.io/confluent-community-license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 */

package io.confluent.kafka.secretregistry.tools;

import java.io.IOException;

import io.confluent.common.utils.AbstractPerformanceTest;
import io.confluent.common.utils.PerformanceStats;
import io.confluent.kafka.secretregistry.client.rest.RestService;
import io.confluent.kafka.secretregistry.client.rest.exceptions.RestClientException;

public class SecretRegistryPerformance extends AbstractPerformanceTest {

  long targetRegisteredSecrets;
  long targetSecretsPerSec;
  String baseUrl;
  RestService restService;
  String path;
  long registeredSecrets = 0;
  long successfullyRegisteredSecrets = 0;

  public static void main(String[] args) throws Exception {

    if (args.length < 4) {
      System.out.println(
          "Usage: java " + SecretRegistryPerformance.class.getName() + " secret_registry_url"
              + " path num_secrets target_secrets_per_sec"
      );
      System.exit(1);
    }

    String baseUrl = args[0];
    String path = args[1];
    int numSecrets = Integer.parseInt(args[2]);
    int targetSecretsPerSec = Integer.parseInt(args[3]);

    SecretRegistryPerformance perf =
        new SecretRegistryPerformance(baseUrl, path, numSecrets, targetSecretsPerSec);
    perf.run(targetSecretsPerSec);
    perf.close();
  }

  public SecretRegistryPerformance(
      String baseUrl, String path, long numSecrets,
      long targetSecretsPerSec
  ) throws Exception {
    super(numSecrets);
    this.baseUrl = baseUrl;
    this.restService = new RestService(baseUrl);
    this.path = path;
    this.targetRegisteredSecrets = numSecrets;
    this.targetSecretsPerSec = targetSecretsPerSec;
  }

  // sequential secret maker
  private static String makeSecret(long num) {
    String secretString = "{\"type\":\"record\","
        + "\"name\":\"myrecord\","
        + "\"fields\":"
        + "[{\"type\":\"string\",\"name\":"
        + "\"f" + num + "\"}]}";
    return secretString;
  }

  @Override
  protected void doIteration(PerformanceStats.Callback cb) {
    String secret = makeSecret(this.registeredSecrets);
    String key = "key";

    try {
      restService.registerSecret(secret, key, this.path);
      successfullyRegisteredSecrets++;
    } catch (IOException e) {
      System.out.println("Problem registering secret: " + e.getMessage());
    } catch (RestClientException e) {
      System.out.println("Problem registering secret: " + e.getMessage());
    }

    registeredSecrets++;
    cb.onCompletion(1, 0);
  }

  protected void close() {
    // We can see some failures due to things like timeouts, but we want it to be obvious
    // if there are too many failures (indicating a real underlying problem). 1% is an arbitrarily
    // chosen limit.
    if (successfullyRegisteredSecrets / (double) targetRegisteredSecrets < 0.99) {
      throw new RuntimeException("Too many secret registration errors: "
          + successfullyRegisteredSecrets
          + " registered successfully out of " + targetRegisteredSecrets
          + " attempted");
    }
  }

  @Override
  protected boolean finished(int iteration) {
    return this.targetRegisteredSecrets == this.registeredSecrets;
  }

  @Override
  protected boolean runningFast(int iteration, float elapsed) {
    return iteration / elapsed > targetSecretsPerSec;
  }
}

